/* For license details see bottom.
 * Copyright (c) 2002 Catalyst of Design (David Morris-Oliveros).  All rights reserved.
 */

// system includes
#include <caosGL/core/globals.h>
#include <caosGL/core/types.h>

// package includes

// extern includes
#include <caosGL/core/Dyngl.h>
#include <GL/glu.h>

#include <caosGL/core/cGLTexture.h>

static log4cpp::Category& cat = log4cpp::Category::getInstance ("caosGL::core::cGLTexture");

namespace caosGL {
	namespace core {
		/**
		 *<br> class:		cGLTexture
		 *<br> namespace:	caosGL::core
		 *<br> inherits:	<none>
		 *<br> implements:	<none>
		 *<br> purpose:		encapsulates an openGL texture
		 *
		 */

		/********************************************************************************************/
		cGLTexture::cGLTexture (const string & name, const eGLTextureType type) {
			_name = name;
			_type = type;
			_generated = false;
			glGenTextures (1, _openGLTexture);
			cat.debug ("Preparing texture for openGL... [%u][%s]", _openGLTexture[0], _name.c_str());
		}
		
		/********************************************************************************************/
		cGLTexture::~cGLTexture () {
			glDeleteTextures (1, _openGLTexture);
		}
		
		/********************************************************************************************/
		tVoid cGLTexture::generate () {
			cat.debug ("Generating : [%u][%s]", _openGLTexture[0], _name.c_str());
			switch (_type) {
			case eGLTextureType_Nearest: 
				// Create Nearest Filtered Texture
				glBindTexture(GL_TEXTURE_2D, _openGLTexture[0]);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
				_generated = true;
			break;
			
			case eGLTextureType_Linear:
				// Create Linear Filtered Texture*/
				glBindTexture(GL_TEXTURE_2D, _openGLTexture[0]);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
				_generated = true;
			break;

			case eGLTextureType_Mipmap:
				// Create MipMapped Texture
				glBindTexture(GL_TEXTURE_2D, _openGLTexture[0]);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
				_generated = true;
			break;
			default:
				cat.error ("Falling to the default level, this cannot happen!");
			break;
			}
		}

		/********************************************************************************************/
		tVoid cGLTexture::prepare () {
			if (_generated) {
				glBindTexture(GL_TEXTURE_2D, _openGLTexture[0]);
			} else {
				cat.error ("Calling prepare on a non-generated texture: [%u][%s]", _openGLTexture[0], _name.c_str());
			}
		}
		
		/********************************************************************************************/
		const string & cGLTexture::name () const {
			return _name;
		}
		
		/********************************************************************************************/
		tVoid cGLTexture::name (const string & name) {
			_name = name;
		}
	}
}

/**
 * The Catalyst of Design Software License, Version 1.0
 *
 * Copyright (c) 2002 Catalyst of Design (David Morris-Oliveros).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by 
 *        Catalyst of Design (http://talsit.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "caosGL" and "Catalyst of Design" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact caosGL@talsit.org.
 *
 * 5. Products derived from this software may not be called "caosGL",
 *    nor may "caosGL" appear in their name, without prior written
 *    permission of Catalyst of Design.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL CATALYST OF DESIGN OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
// eof